<?php

/**
 * @package     Extly.Components
 * @subpackage  com_autotweet - AutoTweetNG posts content to social channels (Twitter, Facebook, LinkedIn, etc).
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2013 Prieco, S.A. All rights reserved.
 * @license     http://http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

include_once 'default.php';

/**
 * AutotweetControllerChannels
 *
 * @package     Extly.Components
 * @subpackage  com_autotweet
 * @since       1.0
*/
class AutotweetControllerChannels extends AutotweetControllerDefault
{
	// Facebook Params
	private $_app_id = null;

	private $_secret = null;

	private $_access_token = null;

	private $_ownapp = null;

	private $_channel_id = null;

	private $_channel_access_token = null;

	// LinkedIn Params
	private $_api_key = null;

	private $_secret_key = null;

	private $_oauth_user_token = null;

	private $_oauth_user_secret = null;

	/**
	 * copy.
	 *
	 * @return	void
	 *
	 * @since	1.5
	 */
	public function authorizedCopy()
	{
		$user = JFactory::getUser();

		if (!$user->authorise('core.create', 'com_autotweet'))
		{
			return JError::raiseError(403, JText::_('JERROR_ALERTNOAUTHOR'));
		}

		$model = $this->getThisModel();
		$status = parent::authorizedCopy($model);

		// Redirect
		$option = $this->input->get('option', 'com_autotweet', 'cmd');
		$view = $this->input->get('view', 'cpanels', 'cmd');
		$url = 'index.php?option=' . $option . '&view=' . $view;

		if (!$status)
		{
			$this->setRedirect($url, $model->getError(), 'error');
		}
		else
		{
			$this->setRedirect($url);
		}

		$this->redirect();
	}

	/**
	 * getParamsForm.
	 *
	 * @return	void
	 */
	public function getParamsForm()
	{
		@ob_end_clean();
		header('Content-type: text/plain');

		// No JInputJSON in J2.5
		$raw = file_get_contents('php://input');
		$data = json_decode($raw, true);

		$token = $data['token'];
		$this->input->set($token, 1);

		// CSRF prevention
		if ($this->csrfProtection)
		{
			$this->_csrfProtection();
		}

		$channeltype_id = $data['channelTypeId'];
		$channel_id = $data['channelId'];

		// Load the model
		$channeltype = FOFModel::getTmpInstance('Channeltypes', 'AutoTweetModel');

		$view = 'nochannel';

		if ($channeltype_id)
		{
			$view = $channeltype->getParamsForm($channeltype_id);
		}

		$config = array(
				'input' => array(
						'option' => 'com_autotweet',
						'view' => $view,
						'task' => (empty($channel_id) ? 'add' : 'edit'),
						'id' => $channel_id,
						'channeltype_id' => $channeltype_id
				),
				'modelName' => 'AutotweetModelChannels'
		);

		@ob_start();
		FOFDispatcher::getTmpInstance('com_autotweet', $view, $config)->dispatch();
		$result = ob_get_contents();
		@ob_end_clean();

		$message = json_encode(
				array(
						'status' => true,
						'message' => $result
				)
		);
		echo EJSON_START . $message . EJSON_END;
		flush();

		JFactory::getApplication()->close();
	}

	/**
	 * getFbValidation.
	 *
	 * @return	void
	 */
	public function getFbValidation()
	{
		@ob_end_clean();
		header('Content-type: text/plain');

		$this->_loadFbParams();

		JLoader::register('FbAppHelper', JPATH_AUTOTWEET_HELPERS . '/channels/fbapp.php');

		$status = false;
		$error_message = 'Unknown';
		$user = null;

		try
		{
			$fbAppHelper = new FbAppHelper($this->_app_id, $this->_secret, $this->_access_token);

			if ($fbAppHelper->login())
			{
				$user = $fbAppHelper->getUser();
				$result = $fbAppHelper->verify();

				$status = $result[0];
				$error_message = $result[1];
			}
			else
			{
				$error_message = 'Facebook Login Failed!';
			}
		}
		catch (Exception $e)
		{
			$error_message = $e->__toString();
		}

		$message = json_encode(
				array(
						'status' => $status,
						'error_message' => $error_message,
						'user' => $user)
		);

		echo EJSON_START . $message . EJSON_END;
		flush();

		JFactory::getApplication()->close();
	}

	/**
	 * getLiValidation.
	 *
	 * @return	void
	 */
	public function getLiValidation()
	{
		@ob_end_clean();
		header('Content-type: text/plain');

		// No JInputJSON in J2.5
		$raw = file_get_contents('php://input');
		$data = json_decode($raw, true);

		$token = $data['token'];
		$this->input->set($token, 1);

		// CSRF prevention
		if ($this->csrfProtection)
		{
			$this->_csrfProtection();
		}

		$api_key = $data['api_key'];
		$secret_key = $data['secret_key'];
		$oauth_user_token = $data['oauth_user_token'];
		$oauth_user_secret = $data['oauth_user_secret'];

		JLoader::register('LiAppHelper', JPATH_AUTOTWEET_HELPERS . '/channels/liapp.php');

		$status = false;
		$error_message = 'Unknown';
		$user = null;

		try
		{
			$appHelper = new LiAppHelper($api_key, $secret_key, $oauth_user_token, $oauth_user_secret);

			if ($appHelper->login())
			{
				$result = $appHelper->getUser();

				if (($result) && (count($result) == 3) && ($result[0]))
				{
					$status = true;
					$error_message = $result[1];
					$user = $result[2];
				}
				else
				{
					$error_message = $result[1];
				}
			}
		}
		catch (Exception $e)
		{
			$error_message = $e->__toString();
		}

		$message = json_encode(
				array(
						'status' => $status,
						'error_message' => $error_message,
						'user' => $user)
		);

		echo EJSON_START . $message . EJSON_END;
		flush();

		JFactory::getApplication()->close();
	}

	/**
	 * getTwValidation.
	 *
	 * @return	void
	 */
	public function getTwValidation()
	{
		@ob_end_clean();
		header('Content-type: text/plain');

		// No JInputJSON in J2.5
		$raw = file_get_contents('php://input');
		$data = json_decode($raw, true);

		$token = $data['token'];
		$this->input->set($token, 1);

		// CSRF prevention
		if ($this->csrfProtection)
		{
			$this->_csrfProtection();
		}

		$consumer_key = $data['consumer_key'];
		$consumer_secret = $data['consumer_secret'];
		$access_token = $data['access_token'];
		$access_token_secret = $data['access_token_secret'];

		JLoader::register('TwAppHelper', JPATH_AUTOTWEET_HELPERS . '/channels/twapp.php');

		$status = false;
		$error_message = 'Unknown';
		$user = null;

		try
		{
			$appHelper = new TwAppHelper($consumer_key, $consumer_secret, $access_token, $access_token_secret);

			if ($result = $appHelper->verify())
			{
				if (count($result) == 3)
				{
					$status = $result[0];
					$error_message = $result[1];
					$user = $result[2];
				}
				else
				{
					$status = $result[0];
					$error_message = $result[1];
				}
			}
			else
			{
				$error_message = 'Twitter Login Failed!';
			}
		}
		catch (Exception $e)
		{
			$error_message = $e->__toString();
		}

		$message = json_encode(
				array(
						'status' => $status,
						'error_message' => $error_message,
						'user' => $user)
		);

		echo EJSON_START . $message . EJSON_END;
		flush();

		JFactory::getApplication()->close();
	}

	/**
	 * getFbChannels.
	 *
	 * @return	void
	 */
	public function getFbChannels()
	{
		@ob_end_clean();
		header('Content-type: text/plain');

		$this->_loadFbParams();

		JLoader::register('FbAppHelper', JPATH_AUTOTWEET_HELPERS . '/channels/fbapp.php');

		$status = false;
		$error_message = 'Unknown';

		try
		{
			$fbAppHelper = new FbAppHelper($this->_app_id, $this->_secret, $this->_access_token);

			if ($fbAppHelper->login())
			{
				$channels = $fbAppHelper->getChannels();
				$status = true;
				$error_message = 'Ok';
			}
			else
			{
				$error_message = 'Facebook Login Failed!';
			}
		}
		catch (Exception $e)
		{
			$error_message = $e->__toString();
		}

		$message = json_encode(
				array(
						'status' => $status,
						'error_message' => $error_message,
						'channels' => $channels)
		);

		echo EJSON_START . $message . EJSON_END;
		flush();

		JFactory::getApplication()->close();
	}

	/**
	 * getFbAlbums.
	 *
	 * @return	void
	 */
	public function getFbAlbums()
	{
		@ob_end_clean();
		header('Content-type: text/plain');

		$this->_loadFbParams();

		JLoader::register('FbAppHelper', JPATH_AUTOTWEET_HELPERS . '/channels/fbapp.php');

		$status = false;
		$error_message = 'Unknown';

		try
		{
			$fbAppHelper = new FbAppHelper($this->_app_id, $this->_secret, $this->_access_token);

			if ($fbAppHelper->login())
			{
				// $this->_channel_access_token ?
				$channels = $fbAppHelper->getAlbums($this->_channel_id);
				$status = true;
				$error_message = 'Ok';
			}
			else
			{
				$error_message = 'Facebook Login Failed!';
			}
		}
		catch (Exception $e)
		{
			$error_message = $e->__toString();
		}

		$message = json_encode(
				array(
						'status' => $status,
						'error_message' => $error_message,
						'albums' => $channels)
		);

		echo EJSON_START . $message . EJSON_END;
		flush();

		JFactory::getApplication()->close();
	}

	/**
	 * getLiGroups
	 *
	 * @return	void
	 */
	public function getLiGroups()
	{
		@ob_end_clean();
		header('Content-type: text/plain');

		$this->_loadLiParams();

		JLoader::register('LiAppHelper', JPATH_AUTOTWEET_HELPERS . '/channels/liapp.php');

		$status = false;
		$error_message = 'Unknown';

		try
		{
			$liAppHelper = new LiAppHelper(
					$this->_api_key,
					$this->_secret_key,
					$this->_oauth_user_token,
					$this->_oauth_user_secret
				);

			if ($liAppHelper->login())
			{
				$channels = $liAppHelper->getMyGroups();
				$status = true;
				$error_message = 'Ok';
			}
			else
			{
				$error_message = 'LiGroups Login Failed!';
			}
		}
		catch (Exception $e)
		{
			$error_message = $e->__toString();
		}

		if ((count($channels) == 2) && ($channels[0] == false))
		{
			$message = json_encode(
					array(
							'status' => false,
							'error_message' => $channels[1])
			);
		}
		else
		{
			$message = json_encode(
					array(
							'status' => $status,
							'error_message' => $error_message,
							'channels' => $channels)
			);
		}

		echo EJSON_START . $message . EJSON_END;
		flush();

		JFactory::getApplication()->close();
	}

	/**
	 * getLiCompanies
	 *
	 * @return	void
	 */
	public function getLiCompanies()
	{
		@ob_end_clean();
		header('Content-type: text/plain');

		$this->_loadLiParams();

		JLoader::register('LiAppHelper', JPATH_AUTOTWEET_HELPERS . '/channels/liapp.php');

		$status = false;
		$error_message = 'Unknown';

		try
		{
			$liAppHelper = new LiAppHelper(
					$this->_api_key,
					$this->_secret_key,
					$this->_oauth_user_token,
					$this->_oauth_user_secret
			);

			if ($liAppHelper->login())
			{
				$channels = $liAppHelper->getMyCompanies();
				$status = true;
				$error_message = 'Ok';
			}
			else
			{
				$error_message = 'LiCompanies Login Failed!';
			}
		}
		catch (Exception $e)
		{
			$error_message = $e->__toString();
		}

		if ((count($channels) == 2) && ($channels[0] == false))
		{
			$message = json_encode(
					array(
							'status' => false,
							'error_message' => $channels[1])
			);
		}
		else
		{
			$message = json_encode(
					array(
							'status' => $status,
							'error_message' => $error_message,
							'channels' => $channels)
			);
		}

		echo EJSON_START . $message . EJSON_END;
		flush();

		JFactory::getApplication()->close();
	}

	/**
	 * _loadFbParams.
	 *
	 * @return	void
	 */
	private function _loadFbParams()
	{
		// No JInputJSON in J2.5
		$raw = file_get_contents('php://input');
		$data = json_decode($raw, true);

		$token = $data['token'];
		$this->input->set($token, 1);

		// CSRF prevention
		if ($this->csrfProtection)
		{
			$this->_csrfProtection();
		}

		$this->_app_id = $data['app_id'];
		$this->_secret = $data['secret'];
		$this->_access_token = $data['access_token'];
		$this->_ownapp = $data['own_app'];
		$this->_channel_id = $data['channel_id'];
		$this->_channel_access_token = $data['channel_access_token'];

		if (!$this->_ownapp)
		{
			$channeltype = FOFModel::getTmpInstance('Channeltypes', 'AutoTweetModel')->getTable();
			$channeltype->reset();
			$channeltype->load(AutotweetModelChanneltypes::TYPE_FBCHANNEL);

			$this->_app_id = $channeltype->auth_key;
			$this->_secret = $channeltype->auth_secret;
		}
	}

	/**
	 * _loadLiParams.
	 *
	 * @return	void
	 */
	private function _loadLiParams()
	{
		// No JInputJSON in J2.5
		$raw = file_get_contents('php://input');
		$data = json_decode($raw, true);

		$token = $data['token'];
		$this->input->set($token, 1);

		// CSRF prevention
		if ($this->csrfProtection)
		{
			$this->_csrfProtection();
		}

		$this->_api_key = $data['api_key'];
		$this->_secret_key = $data['secret_key'];
		$this->_oauth_user_token = $data['oauth_user_token'];
		$this->_oauth_user_secret = $data['oauth_user_secret'];
	}
}
