<?php

/**
 * @package     Extly.Components
 * @subpackage  com_autotweet - AutoTweetNG posts content to social channels (Twitter, Facebook, LinkedIn, etc).
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2013 Prieco, S.A. All rights reserved.
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

/**
 * AutoTweetToolbar
 *
 * @package     Extly.Components
 * @subpackage  com_autotweet
 * @since       1.0
*/
class AutotweetToolbar extends FOFToolbar
{
	protected $isModule = false;

	protected $isModal = false;

	/**
	 * Class constructor
	 *
	 * @param   array  $config  Configuration parameters
	 */
	public function __construct($config = array())
	{
		parent::__construct($config);

		$layout = $this->input->get('layout', null, 'cmd');
		$this->isModule = ($layout == 'module');
		$this->isModal = ($layout == 'modal');
	}

	/**
	 * getMyViews.
	 *
	 * @return	array
	 *
	 * @since	1.5
	 */
	protected function getMyViews()
	{
		if (AUTOTWEETNG_JOOCIAL)
		{
			if (EParameter::getComponentParam(CAUTOTWEETNG, 'targeting', false))
			{
				$views = array(
						'cpanels',
						'requests',
						'rules',
						'posts',
						'channels',
						'feeds',
						'targets',
						'infos'
				);
			}
			else
			{
				$views = array(
						'cpanels',
						'requests',
						'rules',
						'posts',
						'channels',
						'feeds',
						'infos'
				);
			}

			return $views;
		}

		$views = array(
				'cpanels',
				'requests',
				'rules',
				'posts',
				'channels',
				'infos'
		);

		return $views;
	}

	/**
	 * onCpanelBrowse.
	 *
	 * @return	void
	 *
	 * @since	1.5
	 */
	public function onCpanelsBrowse()
	{
		$this->_onAllPages();
		JToolBarHelper::title(VersionHelper::getTitle(JText::_('COM_AUTOTWEET_TITLE_CPANELS')), 'autotweet');
		JToolBarHelper::preferences('com_autotweet', '600', '900');
	}

	/**
	 * onPostsBrowse.
	 *
	 * @return	void
	 *
	 * @since	1.5
	 */
	public function onPostsBrowse()
	{
		if (!$this->isModule)
		{
			$this->_onAllPages();
			JToolBarHelper::title(VersionHelper::getTitle(JText::_('COM_AUTOTWEET_TITLE_POSTS')), 'autotweet-logo.png');

			// Add toolbar buttons
			if ($this->perms->create)
			{
				JToolBarHelper::addNew();
			}

			if ($this->perms->edit)
			{
				JToolBarHelper::editList();
			}

			if ($this->perms->create || $this->perms->edit)
			{
				JToolBarHelper::divider();
			}

			if ($this->perms->editstate)
			{
				JToolBarHelper::publishList();

				// JToolBarHelper::unpublishList();

				JToolBarHelper::divider();
			}

			if ($this->perms->create)
			{
				JToolBarHelper::custom('copy', 'copy.png', 'copy_f2.png', 'COM_AUTOTWEET_COMMON_COPY_LABEL', false);
				JToolBarHelper::divider();
			}

			if ($this->perms->delete)
			{
				$option = $this->input->get('option', 'com_autotweet', 'cmd');
				$msg = JText::_($option . '_CONFIRM_DELETE');
				JToolBarHelper::deleteList($msg);
			}
		}
	}

	/**
	 * onRequestsBrowse.
	 *
	 * @return	void
	 *
	 * @since	1.5
	 */
	public function onRequestsBrowse()
	{
		$this->_onAllPages();
		JToolBarHelper::title(VersionHelper::getTitle(JText::_('COM_AUTOTWEET_TITLE_REQUESTS')), 'autotweet-logo.png');

		if ($this->perms->create)
		{
			JToolBarHelper::addNew();
		}

		if ($this->perms->edit)
		{
			JToolBarHelper::editList();
		}

		if ($this->perms->create || $this->perms->edit)
		{
			JToolBarHelper::divider();
		}

		if ($this->perms->editstate)
		{
			JToolBarHelper::custom('process', 'process.png', 'process.png', 'COM_AUTOTWEET_COMMON_PROCESS_LABEL', false);
			JToolBarHelper::divider();
		}

		if ($this->perms->create)
		{
			JToolBarHelper::custom('copy', 'copy.png', 'copy_f2.png', 'COM_AUTOTWEET_COMMON_COPY_LABEL', false);
			JToolBarHelper::divider();
		}

		if ($this->perms->delete)
		{
			$option = $this->input->get('option', 'com_autotweet', 'cmd');
			$msg = JText::_($option . '_CONFIRM_DELETE');
			JToolBarHelper::deleteList($msg);
		}
	}

	/**
	 * onRequests.
	 *
	 * @return	void
	 *
	 * @since	1.5
	 */
	public function onRequests()
	{
		parent::onAdd();
		JToolBarHelper::title(VersionHelper::getTitle(JText::_('COM_AUTOTWEET_TITLE_REQUEST_EDIT')), 'autotweet-logo.png');
	}

	/**
	 * onChannelsBrowse.
	 *
	 * @return	void
	 *
	 * @since	1.5
	 */
	public function onChannelsBrowse()
	{
		$this->_onAllPages();
		JToolBarHelper::title(VersionHelper::getTitle(JText::_('COM_AUTOTWEET_TITLE_CHANNELS')), 'autotweet-logo.png');
		$this->_onBrowseWithCopy();
	}

	/**
	 * onChannels.
	 *
	 * @return	void
	 *
	 * @since	1.5
	 */
	public function onChannels()
	{
		parent::onAdd();
		JToolBarHelper::title(VersionHelper::getTitle(JText::_('COM_AUTOTWEET_TITLE_CHANNELS_EDIT')), 'autotweet-logo.png');
	}

	/**
	 * onRulesBrowse.
	 *
	 * @return	void
	 *
	 * @since	1.5
	 */
	public function onRulesBrowse()
	{
		$this->_onAllPages();
		JToolBarHelper::title(VersionHelper::getTitle(JText::_('COM_AUTOTWEET_TITLE_RULES')), 'autotweet-logo.png');
		$this->_onBrowseWithCopy();
	}

	/**
	 * onRules.
	 *
	 * @return	void
	 *
	 * @since	1.5
	 */
	public function onRules()
	{
		parent::onAdd();
		JToolBarHelper::title(VersionHelper::getTitle(JText::_('COM_AUTOTWEET_TITLE_RULE_EDIT')), 'autotweet-logo.png');
	}

	/**
	 * onPostBrowse.
	 *
	 * @return	void
	 *
	 * @since	1.5
	 */
	public function onPostBrowse()
	{
		$this->_onAllPages();
		JToolBarHelper::title(VersionHelper::getTitle(JText::_('COM_AUTOTWEET_TITLE_POST')), 'autotweet-logo.png');
	}

	/**
	 * onPosts.
	 *
	 * @return	void
	 *
	 * @since	1.5
	 */
	public function onPosts()
	{
		parent::onAdd();
		JToolBarHelper::title(VersionHelper::getTitle(JText::_('COM_AUTOTWEET_TITLE_POST_EDIT')), 'autotweet-logo.png');
	}

	/**
	 * onTargetsBrowse.
	 *
	 * @return	void
	 *
	 * @since	1.5
	 */
	public function onTargetsBrowse()
	{
		JToolBarHelper::title(VersionHelper::getTitle(JText::_('COM_AUTOTWEET_TITLE_TARGETS')), 'autotweet-logo.png');

		if (!$this->isModal)
		{
			$this->_onAllPages();
			$this->_onBrowseWithCopy();
		}
	}

	/**
	 * onTargets.
	 *
	 * @return	void
	 *
	 * @since	1.5
	 */
	public function onTargets()
	{
		parent::onAdd();
		JToolBarHelper::title(VersionHelper::getTitle(JText::_('COM_AUTOTWEET_TITLE_TARGETS_EDIT')), 'autotweet-logo.png');
	}

	/**
	 * onFeedsBrowse.
	 *
	 * @return	void
	 *
	 * @since	1.5
	 */
	public function onFeedsBrowse()
	{
		JToolBarHelper::title(VersionHelper::getTitle(JText::_('COM_AUTOTWEET_TITLE_FEEDS')), 'autotweet-logo.png');
		$this->_onAllPages();
		$this->_onBrowseWithCopy();
		JToolBarHelper::custom('import', 'process.png', 'process.png', 'COM_AUTOTWEET_COMMON_IMPORT_LABEL', false);
	}

	/**
	 * onFeedsAdd.
	 *
	 * @return	void
	 *
	 * @since	1.5
	 */
	public function onFeeds()
	{
		// On frontend, buttons must be added specifically
		if (!FOFPlatform::getInstance()->isBackend() && !$this->renderFrontendButtons)
		{
			return;
		}

		$option = $this->input->getCmd('option', 'com_foobar');
		$componentName = str_replace('com_', '', $option);

		// Set toolbar title
		$subtitle_key = strtoupper($option . '_TITLE_' . FOFInflector::pluralize($this->input->getCmd('view', 'cpanel'))) . '_EDIT';
		JToolBarHelper::title(JText::_(strtoupper($option)) . ' &ndash; <small>' . JText::_($subtitle_key) . '</small>', $componentName);

		// Set toolbar icons
		$bar = JToolbar::getInstance('toolbar');

		// Add an 'Apply & Preview' button
		$bar->appendButton('Standard', 'apply', 'COM_AUTOTWEET_VIEW_FEED_PREVIEW_JTOOLBAR_APPLY', 'apply', false);

		JToolBarHelper::save();
		JToolBarHelper::custom('savenew', 'save-new.png', 'save-new_f2.png', 'JTOOLBAR_SAVE_AND_NEW', false);
		JToolBarHelper::custom('import', 'process.png', 'process.png', 'COM_AUTOTWEET_COMMON_IMPORT_LABEL', false);
		JToolBarHelper::cancel();

		JToolBarHelper::title(VersionHelper::getTitle(JText::_('COM_AUTOTWEET_TITLE_FEEDS_EDIT')), 'autotweet-logo.png');
	}

	/**
	 * onInfosBrowse.
	 *
	 * @return	void
	 *
	 * @since	1.5
	 */
	public function onInfosBrowse()
	{
		$this->_onAllPages();
		JToolBarHelper::title(VersionHelper::getTitle(JText::_('COM_AUTOTWEET_TITLE_INFOS')), 'autotweet-logo.png');
		JToolBarHelper::preferences('com_autotweet', '600', '900');
	}

	/**
	 * _onAllPages.
	 *
	 * @return	void
	 *
	 * @since	1.5
	 */
	public function _onAllPages()
	{
		// On frontend, buttons must be added specifically
		list($isCli, $isAdmin) = FOFDispatcher::isCliAdmin();

		if ($isAdmin || $this->renderFrontendSubmenu)
		{
			$this->renderSubmenu();
		}

		if (!$isAdmin && !$this->renderFrontendButtons)
		{
			return;
		}
	}

	/**
	 * _onBrowseWithCopy.
	 *
	 * @param   bool  $allowCopy  Param.
	 *
	 * @return	void
	 *
	 * @since	1.5
	 */
	public function _onBrowseWithCopy($allowCopy = true)
	{
		// Add toolbar buttons
		if ($this->perms->create)
		{
			JToolBarHelper::addNew();
		}

		if ($this->perms->edit)
		{
			JToolBarHelper::editList();
		}

		if ($this->perms->create || $this->perms->edit)
		{
			JToolBarHelper::divider();
		}

		if ($this->perms->editstate)
		{
			JToolBarHelper::publishList();
			JToolBarHelper::unpublishList();
			JToolBarHelper::divider();
		}

		if (($allowCopy) && ($this->perms->create))
		{
			JToolBarHelper::custom('copy', 'copy.png', 'copy_f2.png', 'COM_AUTOTWEET_COMMON_COPY_LABEL', false);
			JToolBarHelper::divider();
		}

		if ($this->perms->delete)
		{
			$option = $this->input->get('option', 'com_autotweet', 'cmd');
			$msg = JText::_($option . '_CONFIRM_DELETE');
			JToolBarHelper::deleteList($msg);
		}
	}
}
