<?php

/**
 * @package     Extly.Library
 * @subpackage  lib_extly - Extly Framework
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2012 Prieco, S.A. All rights reserved.
 * @license     http://http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

/**
 * This is the base class for the Extly framework.
 *
 * @package     Extly.Library
 * @subpackage  lib_extly
 * @since       1.0
*/
class Extly
{
	private static $_version = '1.0.0';

	private static $_appAvailable = false;

	private static $_isRendered = false;

	private static $_ownJqueryDisabled25 = false;
	const JS_LIB = 'media/lib_extly/js/';
	const JS_BEGIN_COMMENT = "\n<!-- EXTLY BEGIN -->\n";
	const JS_END_COMMENT = "\n<!-- EXTLY END -->\n";
	const JS_SOBIPRO10_ONREADY = "SobiPro.onReady";
	const JS_SOBIPRO11_ONREADY = "SobiPro.jQuery( document ).ready";
	const JS_BODY = '</body>';
	const JS_HTML = '</html>';
	const JS_SCRIPT_BEGIN = "\n\n<script>/*\n<![CDATA[*/\n\n";
	const JS_SCRIPT_END = "\n/*]]>*/</script>\n\n";
	const JS_MOO_INIT = <<<JS
	<!-- Extly Dependency Manager -->

		var allKeys, config = {
			baseUrl: '{DATA_URLBASE}',
			urlArgs: 'bust={DATA_VERSION}',
			shim: {},
			paths: {}
		};

		// Additional dependencies
		xtDependencies.each(function(dependency) {
	  		config.shim[dependency.key] = dependency.value;
		});

		// Additional paths
		allKeys = [ 'extlycore' ];
		xtPaths.each(function(path) {
			allKeys.push(path.key);
	  		config.paths[path.key] = path.value;
		});

		// Require.js allows us to configure shortcut alias
		require.config(config);

		require(allKeys, function(Core) {

		});

JS;
	const JS_JQ_INIT = <<<JS
	<!-- Extly Dependency Manager -->

		var allKeys, config = {
			baseUrl: '{DATA_URLBASE}',
			urlArgs: 'bust={DATA_VERSION}',
			shim: {},
			paths: {}
		};

		// Additional dependencies
		jQuery.each(xtDependencies, function(index, dependency) {
	  		config.shim[dependency.key] = dependency.value;
		});

		// Additional paths
		allKeys = [ 'extlycore' ];
		jQuery.each(xtPaths, function(index, path) {
			allKeys.push(path.key);
	  		config.paths[path.key] = path.value;
		});

		// Require.js allows us to configure shortcut alias
		require.config(config);

		require(allKeys, function(Core) {

		});

JS;

	/**
	 * loadMeta.
	 *
	 * @return	void
	 *
	 * @since	1.0
	 */
	public static function loadMeta()
	{
		$document = JFactory::getDocument();
		$document->setMetaData('X-UA-Compatible', 'IE=edge,chrome=1');
	}

	/**
	 * loadStyle.
	 *
	 * @return	void
	 *
	 * @since	1.0
	 */
	public static function loadStyle()
	{
		// Joomla 2.5
		if (version_compare(JVERSION, '3.0.0', 'lt'))
		{
			JHtml::stylesheet('lib_extly/extly-bootstrap.min.css', false, true);
			JHtml::stylesheet('lib_extly/bootstrap-extended.css', false, true);
			JHtml::stylesheet('lib_extly/chosen.css', false, true);
		}
		else
		{
			// Joomla 3.0 or superior
			// Assumed bootstrap.framework
		}

		JHtml::stylesheet('lib_extly/extly-base660.css', false, true);
	}

	/**
	 * loadStyle.
	 *
	 * @return	void
	 *
	 * @since	1.0
	 */
	public static function loadAwesome()
	{
		JHtml::stylesheet('lib_extly/extly-font-awesome.min.css', false, true);
	}

	/**
	 * loadHtml
	 *
	 * @return	void
	 *
	 * @since	1.0
	 */
	public static function loadHtml()
	{
		JHtml::addIncludePath(JPATH_ROOT . '/libraries/extly/helpers/html');
	}

	/**
	 * initApp.
	 *
	 * @param   string  $version              Param
	 * @param   string  $extensionmainjs      Param
	 * @param   array   $dependencies         {key2 => {key1, keyi}}
	 * @param   array   $paths                {key1 => pathjs1, key2 => pathjs2}
	 * @param   bool    $ownJqueryDisabled25  Param
	 *
	 * @return	void
	 *
	 * @since	1.0
	 */
	public static function initApp($version = null, $extensionmainjs = null, $dependencies = array(), $paths = array(), $ownJqueryDisabled25 = false)
	{
		$root = JUri::root();

		self::$_appAvailable = true;
		self::$_version = $version;

		// Module dependencies must be added
		$key = str_replace('.js', '', $extensionmainjs);
		$key = basename($key);

		if ((!array_key_exists($key, $dependencies)) && (!empty($paths)))
		{
			$alldeps = implode('\',\'', array_keys($paths));
			$dependencies [$key] = "{deps: ['" . $alldeps . "']}";
		}

		if (version_compare(JVERSION, '3.0.0', 'lt'))
		{
			// Joomla 2.5
			JHTML::_('behavior.mootools');

			// JQuery - Bootstrap
			if ($ownJqueryDisabled25)
			{
				$paths ['jquery183'] = self::JS_LIB . 'jquery/jquery.disabled';
				self::$_ownJqueryDisabled25 = true;
			}
			else
			{
				$paths ['jquery183'] = self::JS_LIB . 'jquery/jquery.min';
			}

			// Chosen - tooltip
			if (JFactory::getApplication()->isAdmin())
			{
				$dependencies ['bootstrap'] = "{deps: ['jquery183']}";
				$dependencies ['chosen'] = "{deps: ['jquery183']}";
				$dependencies ['j3compat'] = "{deps: ['bootstrap', 'chosen']}";

				$paths ['bootstrap'] = self::JS_LIB . 'bootstrap/bootstrap.min';
				$paths ['chosen'] = self::JS_LIB . 'j3/chosen.jquery.min';
				$paths ['j3compat'] = self::JS_LIB . 'j3compat.min';
			}
		}
		else
		{
			// Joomla 3.0 or superior
			// JQuery - Bootstrap
			JHtml::_('jquery.framework');

			// Chosen - tooltip
			if (JFactory::getApplication()->isAdmin())
			{
				JHtml::_('bootstrap.framework');
				JHtml::_('formbehavior.chosen', 'select');
				JHtml::_('behavior.tooltip');
			}
		}

		self::_initJsApp($extensionmainjs, $dependencies, $paths);
	}

	/**
	 * _initJsApp
	 *
	 * @param   string  $extensionmainjs  Param
	 * @param   array   $dependencies     {key2 => {key1, keyi}}
	 * @param   array   $paths            {key1 => pathjs1, key2 => pathjs2}
	 *
	 * @return	void
	 *
	 * @since	1.0
	 */
	public static function _initJsApp($extensionmainjs = null, $dependencies = array(), $paths = array())
	{
		static $_initialized = false;

		$document = JFactory::getDocument();
		$root = JUri::root();

		$document->addScript($root . self::JS_LIB . 'require/require.min.js');

		$host = self::_getHost();
		$mainjsbase = $host . dirname($extensionmainjs);

		if (!$_initialized)
		{
			$_initialized = true;
			$document->addScriptDeclaration("\nvar xtDependencies = [], xtPaths = [];\n");
		}

		// Dependencies and Paths
		$document->addScriptDeclaration(self::JS_BEGIN_COMMENT);

		if (version_compare(JVERSION, '3.0.0', 'lt'))
		{
			// Joomla 2.5
			$dependencies ['underscore'] = "{deps: ['jquery183'], exports: '_'}";

			if (JFactory::getApplication()->isSite())
			{
				$dependencies ['extlycore'] = "{deps: ['backbone']}";
			}
			else
			{
				$dependencies ['extlycore'] = "{deps: ['backbone', 'j3compat']}";
			}
		}
		else
		{
			// Joomla 3.0 or superior
			$dependencies ['underscore'] = "{exports: '_'}";
			$dependencies ['extlycore'] = "{deps: ['backbone']}";
		}

		$dependencies ['backbone'] = "{deps: ['underscore'], exports: 'Backbone'}";

		$paths ['underscore'] = self::JS_LIB . 'backbone/lodash.min';
		$paths ['backbone'] = self::JS_LIB . 'backbone/backbone.min';

		if (JFactory::getApplication()->isSite())
		{
			$paths ['extlycore'] = self::JS_LIB . 'extlycorefront.min';
		}
		else
		{
			$paths ['extlycore'] = self::JS_LIB . 'extlycore.min';
		}

		$listofdeps = array();

		foreach ($dependencies as $dependency_key => $dependency_value)
		{
			self::generateDeclaration($dependency_key, $dependency_value);
		}

		if ($extensionmainjs)
		{
			$config = JFactory::getConfig();

			if ($config->get('debug'))
			{
				$extensionmainjs = $host . str_replace('.min.js', '', $extensionmainjs);
			}
			else
			{
				$extensionmainjs = $host . str_replace('.js', '', $extensionmainjs);
			}

			$appName = basename($extensionmainjs);
			$document->addScriptDeclaration("xtPaths.push({key:'{$appName}', value: '{$extensionmainjs}'});");
		}

		foreach ($paths as $path_key => $path_value)
		{
			self::generatePath($path_key, $path_value);
		}

		$document->addScriptDeclaration(self::JS_END_COMMENT);
	}

	/**
	 * insertDependencyManager.
	 *
	 * @param   string  &$body  Param
	 *
	 * @return	void
	 */
	public static function insertDependencyManager(&$body)
	{
		if (self::hasApp())
		{
			self::$_isRendered = true;

			$root = JUri::root();

			if (version_compare(JVERSION, '3.0.0', 'lt'))
			{
				// Joomla 2.5
				$jsapp_init = self::JS_MOO_INIT;
			}
			else
			{
				// Joomla 3.0 or superior
				$jsapp_init = self::JS_JQ_INIT;
			}

			$jsapp_init = str_replace('{DATA_URLBASE}', $root, $jsapp_init);
			$jsapp_init = str_replace('{DATA_VERSION}', self::$_version, $jsapp_init);

			$pos = strrpos($body, self::JS_BODY);

			if ($pos !== false)
			{
				$body = substr($body, 0, $pos) . self::JS_SCRIPT_BEGIN . $jsapp_init . self::JS_SCRIPT_END . substr($body, $pos);
			}
			else
			{
				$pos = strrpos($body, self::JS_HTML);

				if ($pos !== false)
				{
					$body = substr($body, 0, $pos) . self::JS_SCRIPT_BEGIN . $jsapp_init . self::JS_SCRIPT_END . substr($body, $pos);
				}
			}
		}
	}

	/**
	 * hasApp
	 *
	 * @return	bool
	 */
	public static function hasApp()
	{
		return ((self::$_appAvailable) && (!self::$_isRendered));
	}

	/**
	 * generateDeclaration
	 *
	 * @param   string  $key    Param
	 * @param   string  $value  Param
	 *
	 * @return	void
	 *
	 * @since	1.0
	 */
	private static function generateDeclaration($key, $value)
	{
		static $jsDependencies = array();

		if (!array_key_exists($key, $jsDependencies))
		{
			$jsDependencies [$key] = $value;
			$document = JFactory::getDocument();
			$document->addScriptDeclaration("xtDependencies.push({key:'{$key}', value:{$value}});");
		}
	}

	/**
	 * generateDeclaration
	 *
	 * @param   string  $key    Param
	 * @param   string  $value  Param
	 *
	 * @return	void
	 *
	 * @since	1.0
	 */
	private static function generatePath($key, $value)
	{
		static $jsPaths = array();

		if (!array_key_exists($key, $jsPaths))
		{
			$jsPaths [$key] = $value;
			$document = JFactory::getDocument();
			$document->addScriptDeclaration("xtPaths.push({key:'{$key}', value: '{$value}'});");
		}
	}

	/**
	 * getFormId.
	 *
	 * @return	void
	 *
	 * @since	1.0
	 */
	public static function getFormId()
	{
		return 'adminForm';
	}

	/**
	 * showInvalidFormAlert.
	 *
	 * @return	void
	 *
	 * @since	1.0
	 */
	public static function showInvalidFormAlert()
	{
		?>
<div id="invalid-form" class="alert alert-block alert-error"
	style="display: none;">
	<button data-dismiss="alert" class="close" type="button">×</button>
	<h4 class="alert-heading">
		<?php echo JText::_('JGLOBAL_VALIDATION_FORM_FAILED'); ?>
	</h4>
</div>
<?php
	}

	/**
	 * getHost.
	 *
	 * @return	string
	 */
	private static function _getHost()
	{
		$baseurl = JUri::root();
		$uri = JUri::getInstance();

		if ($uri->parse($baseurl))
		{
			$host = $uri->toString(
				array(
							'scheme',
							'host',
							'port'
				)
			);

			return $host;
		}

		return null;
	}

	/**
	 * loadComponentLanguage.
	 *
	 * @param   string  $option  Param
	 *
	 * @return	void
	 */
	public static function loadComponentLanguage($option)
	{
		// Component Language Load
		$jlang = JFactory::getLanguage();
		$paths = array(
						JPATH_ADMINISTRATOR,
						JPATH_ROOT
		);
		$jlang->load($option, $paths [0], 'en-GB', true);
		$jlang->load($option, $paths [0], null, true);
		$jlang->load($option, $paths [1], 'en-GB', true);
		$jlang->load($option, $paths [1], null, true);

		return $jlang;
	}
}
