<?php

/**
 * @package     Extly.Library
 * @subpackage  lib_extly - Extly Framework
 *
 * @author      Prieco S.A. <support@extly.com>
 * @copyright   Copyright (C) 2007 - 2012 Prieco, S.A. All rights reserved.
 * @license     http://http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        http://www.extly.com http://support.extly.com
 */
// No direct access
defined('_JEXEC') or die('Restricted access');

jimport('joomla.html.html');

/**
 * Utility class for all HTML drawing classes
 *
 * @package     Extly.Library
 * @subpackage  HTML
 * @since       11.1
*/
abstract class EHtml extends JHtml
{
	/**
	 * Compute the files to be included
	 *
	 * @param   string   $folder          folder name to search into (images, css, js, ...)
	 * @param   string   $file            path to file
	 * @param   boolean  $relative        path to file is relative to /media folder
	 * @param   boolean  $path_only       param
	 * @param   boolean  $detect_browser  detect browser to include specific browser files
	 * @param   boolean  $detect_debug    detect debug to include compressed files if debug is on
	 *
	 * @return  array    files to be included
	 *
	 * @see     JBrowser
	 * @since   11.1
	 */
	public static function getRelativeFiles($folder, $file, $relative = true, $path_only = false, $detect_browser = false, $detect_debug = true)
	{
		return self::includeRelativeFiles($folder, $file, $relative, $detect_browser, $detect_debug);
	}

	/**
	 * generateIdTag
	 *
	 * @return  string
	 */
	public static function generateIdTag()
	{
		$idTag = 'id' . rand(1, 10000);
		$idTag = 'xtform' . $idTag;

		return $idTag;
	}

	/**
	 * genericControl.
	 *
	 * @param   string  $label          Label
	 * @param   string  $desc           Description
	 * @param   array   $name           Control name
	 * @param   string  $control        Params
	 * @param   string  $control_class  Params
	 *
	 * @return  string  HTML
	 */
	public static function genericControl($label, $desc, $name = null, $control = null, $control_class = null)
	{
		return "
		<div class=\"control-group " . $control_class . "\">
		<label for=\"{$name}\" class=\"control-label\" rel=\"tooltip\" data-original-title=\"" . JText::_($desc) . "\">" . JText::_($label) . "</label>
		<div class=\"controls\">" . $control . "</div>
		</div>

		";
	}

	/**
	 * textControl.
	 *
	 * @param   string  $selected   Value
	 * @param   string  $name       The name for the field
	 * @param   string  $label      Label
	 * @param   string  $desc       Description
	 * @param   array   $idTag      Additional HTML attributes for the <select> tag
	 * @param   int     $maxlength  Param
	 *
	 * @return  string  HTML
	 */
	public static function textControl($selected, $name, $label, $desc, $idTag = null, $maxlength = 32)
	{
		if (!$idTag)
		{
			$idTag = self::generateIdTag();
		}

		$mclass = '';

		if ($maxlength < 7)
		{
			$mclass = ' class="input-mini"';
		}

		return "
		<div class=\"control-group\">
		<label for=\"{$idTag}\" class=\"control-label\" rel=\"tooltip\" data-original-title=\"" . JText::_($desc) . "\">" . JText::_($label) . "</label>
		<div class=\"controls\">
		<input type=\"text\" name=\"{$name}\" id=\"{$idTag}\" value=\"{$selected}\" maxlength=\"{$maxlength}\" {$mclass}/>
		</div>
		</div>

		";
	}

	/**
	 * readonlyTextControl.
	 *
	 * @param   string  $selected   Value
	 * @param   string  $name       The name for the field
	 * @param   string  $label      Label
	 * @param   string  $desc       Description
	 * @param   array   $idTag      Additional HTML attributes for the <select> tag
	 * @param   int     $maxlength  Param
	 *
	 * @return  string  HTML
	 */
	public static function readonlyTextControl($selected, $name, $label, $desc, $idTag = null, $maxlength = 32)
	{
		if (!$idTag)
		{
			$idTag = self::generateIdTag();
		}

		$mclass = 'readonly ';

		if ($maxlength < 7)
		{
			$mclass = $mclass . ' class="input-mini"';
		}

		return "
		<div class=\"control-group\">
		<label for=\"{$idTag}\" class=\"control-label\" rel=\"tooltip\" data-original-title=\"" . JText::_($desc) . "\">" . JText::_($label) . "</label>
		<div class=\"controls\">
		<input type=\"text\" name=\"{$name}\" id=\"{$idTag}\" value=\"{$selected}\" maxlength=\"{$maxlength}\" {$mclass} readonly=\"readonly\"/>
		</div>
		</div>

		";
	}

	/**
	 * requiredTextControl.
	 *
	 * @param   string  $selected   Value
	 * @param   string  $name       The name for the field
	 * @param   string  $label      Label
	 * @param   string  $desc       Description
	 * @param   array   $idTag      Additional HTML attributes for the <select> tag
	 * @param   int     $maxlength  Param
	 *
	 * @return  string  HTML
	 */
	public static function requiredTextControl($selected, $name, $label, $desc, $idTag = null, $maxlength = 32)
	{
		if (!$idTag)
		{
			$idTag = self::generateIdTag();
		}

		$mclass = '';

		if ($maxlength < 7)
		{
			$mclass = ' input-mini';
		}

		return "
		<div class=\"control-group\">
		<label for=\"{$idTag}\" class=\"control-label required\" rel=\"tooltip\" data-original-title=\"" . JText::_($desc) . "\">" . JText::_($label) . " <span class=\"star\">&#160;*</span> </label>
		<div class=\"controls\">
		<input type=\"text\" name=\"{$name}\" id=\"{$idTag}\" value=\"{$selected}\" class=\"required{$mclass}\" maxlength=\"{$maxlength}\" required=\"required\" />
		</div>
		</div>

		";
	}

	/**
	 * textareaControl.
	 *
	 * @param   string  $selected  Value
	 * @param   string  $name      The name for the field
	 * @param   string  $label     Label
	 * @param   string  $desc      Description
	 * @param   array   $idTag     Additional HTML attributes for the <select> tag*
	 *
	 * @return  string  HTML
	*/
	public static function textareaControl($selected, $name, $label, $desc, $idTag = null)
	{
		if (!$idTag)
		{
			$idTag = self::generateIdTag();
		}

		return "
		<div class=\"control-group\">
		<label for=\"{$idTag}\" class=\"control-label\" rel=\"tooltip\" data-original-title=\"" . JText::_($desc) . "\">" . JText::_($label) . "</label>
		<div class=\"controls\">
		<textarea id=\"{$idTag}\" rows=\"5\" cols=\"60\"
		name=\"{$name}\">{$selected}</textarea>
		</div>
		</div>

		";
	}

	/**
	 * idControl.
	 *
	 * @param   string  $selected  Value
	 * @param   string  $name      The name for the field
	 * @param   array   $idTag     Additional HTML attributes for the <select> tag*
	 *
	 * @return  string  HTML
	 */
	public static function idControl($selected, $name = 'id', $idTag = null)
	{
		if (!$idTag)
		{
			$idTag = self::generateIdTag();
		}

		return "
		<div class=\"control-group\">
		<label for=\"{$idTag}\" class=\"control-label\" rel=\"tooltip\" data-original-title=\"" . JText::_('JGLOBAL_FIELD_ID_DESC') . "\">" . JText::_('JGLOBAL_FIELD_ID_LABEL') . "</label>
		<div class=\"controls\">
		<input type=\"text\" name=\"{$name}\" id=\"{$idTag}\" value=\"{$selected}\" class=\"uneditable-input\" readonly=\"readonly\">
		</div>
		</div>

		";
	}

	/**
	 * ajaxButtonControl.
	 *
	 * @param   string  $url       Value
	 * @param   string  $selected  Value
	 * @param   string  $name      The name for the field
	 * @param   string  $label     The label for the field
	 * @param   string  $desc      The desc for the field
	 * @param   string  $button    The button for the field
	 * @param   string  $idTag     The id for the field
	 * @param   string  $class     The class for the field
	 *
	 * @return  string  HTML
	 */
	public static function ajaxButtonControl($url, $selected, $name, $label, $desc, $button, $idTag = null, $class = null)
	{
		if (!$idTag)
		{
			$idTag = self::generateIdTag();
		}

		if (version_compare(JVERSION, '3.0.0', 'ge'))
		{
			$span = 'span3';
		}
		else
		{
			$span = 'span8';
		}

		return "
		<div class=\"control-group\">
		<label for=\"{$idTag}\" class=\"control-label\" rel=\"tooltip\" data-original-title=\"" . JText::_($desc) . "\">" . JText::_($label) . "</label>
						<div class=\"controls\">
						<a class=\"btn pull-left xt-ajax-button " . $class . "\" href=\"" . $url . "\">" . JText::_($button) . "</a>&nbsp;
						<input type=\"text\" value=\"" . JText::_($selected) . "\" class=\"" . $span . " xt-ajax-message uneditable-input\" readonly=\"readonly\">
						</div>
						</div>

			";
	}

	/**
	 * downloadButtonControl.
	 *
	 * @param   string  $url       Value
	 * @param   string  $selected  Value
	 * @param   string  $name      The name for the field
	 * @param   string  $label     The label for the field
	 * @param   string  $desc      The desc for the field
	 * @param   string  $button    The button for the field
	 * @param   string  $idTag     The id for the field
	 * @param   string  $class     The class for the field
	 *
	 * @return  string  HTML
	 */
	public static function downloadButtonControl($url, $selected, $name, $label, $desc, $button, $idTag = null, $class = null)
	{
		if (!$idTag)
		{
			$idTag = self::generateIdTag();
		}

		return "
		<div class=\"control-group\">
		<label for=\"{$idTag}\" class=\"control-label\" rel=\"tooltip\" data-original-title=\"" . JText::_($desc) . "\">" . JText::_($label) . "</label>
						<div class=\"controls\">
						<a class=\"btn pull-left " . $class . "\" href=\"" . $url . "\">" . JText::_($button) . "</a>
						</div>
						</div>

			";
	}

	/**
	 * calendarControl.
	 *
	 * @param   string  $selected     Value
	 * @param   string  $name         The name for the field
	 * @param   string  $label        Label
	 * @param   string  $desc         Description
	 * @param   array   $idTag        Additional HTML attributes for the <select> tag
	 * @param   string  $date_format  Date format
	 * @param   string  $class        Class
	 *
	 * @return  string  HTML
	 */
	public static function calendarControl($selected, $name, $label, $desc, $idTag = null, $date_format = null, $class = 'input')
	{
		JHTML::_('behavior.calendar');

		if (empty($date_format))
		{
			$date_format = 'DATE_FORMAT_LC4';
		}

		if (!$idTag)
		{
			$idTag = self::generateIdTag();
		}

		if (!empty($selected))
		{
			$selected = JHtml::_('date', $selected, JText::_($date_format));
		}

		$control = JHTML::_('calendar', $selected, $name, $idTag, '%Y-%m-%d',
				array(
						'class' => $class
			)
		);

		return self::genericControl($label, $desc, $name, $control);
	}

	/**
	 * imageControl.
	 *
	 * @param   string  $selected  Value
	 * @param   string  $name      The name for the field
	 * @param   string  $label     Label
	 * @param   string  $desc      Description
	 * @param   array   $idTag     Additional HTML attributes for the <select> tag
	 * @param   string  $class     Class
	 *
	 * @return  string  HTML
	 */
	public static function imageControl($selected, $name, $label, $desc, $idTag = null, $class = null)
	{
		static $inserted = false;
		JHTML::_('behavior.modal');

		if (!$idTag)
		{
			$idTag = self::generateIdTag();
		}

		if (!empty($class))
		{
			$class .= 'class="' . $class . '"';
		}

		$control = '<input type="text" name="' . $name . '" id="' . $idTag . '" value="' . $selected . '" maxlength="512"
						. ' . $class . '/>
						<a class="btn btn-mini modal" rel="{handler: \'iframe\', size: {x: 800, y: 500}}"
							href="index.php?option=com_media&amp;view=images&amp;tmpl=component&amp;asset=com_autotweet&amp;author=&amp;fieldid=' . $idTag . '" title="' . JText::_('JSELECT') . '">' . JText::_('JSELECT') . '</a>';

		if (!$inserted)
		{
			$inserted = true;

			$document = JFactory::getDocument();
			$document->addScriptDeclaration("\n\n// Extly's imageControl\n\t//
		window.jInsertFieldValue = function jInsertFieldValue(value, id) {
			jQuery('#' + id).val(value);
		};\n\n");
		}

		return self::genericControl($label, $desc, $name, $control);
	}

	/**
	 * userControl.
	 *
	 * @param   string  $selected  Value
	 * @param   string  $name      The name for the field
	 * @param   string  $label     Label
	 * @param   string  $desc      Description
	 * @param   array   $idTag     Additional HTML attributes for the <select> tag
	 * @param   string  $class     Class
	 *
	 * @return  string  HTML
	 */
	public static function userControl($selected, $name, $label, $desc, $idTag = null, $class = null)
	{
		JHTML::_('behavior.modal');

		if (!$idTag)
		{
			$idTag = self::generateIdTag();
		}

		$bogusNameTag = self::generateIdTag();

		if (!empty($class))
		{
			$class .= 'class="' . $class . '"';
		}

		$username = null;
		$user = JFactory::getUser($selected);
		$username = $user->username;

		$control = '<input type="hidden" name="' . $name . '" id="' . $idTag . '" value="' . $selected . '"/>
					<input type="text" class="input-medium" name="' . $bogusNameTag . '" id="' . $idTag . '_username" value="' . $username . '" disabled="disabled" />
						<a class="btn btn-mini modal" rel="{handler: \'iframe\', size: {x: 800, y: 500}}"
							href="index.php?option=com_users&amp;view=users&amp;layout=modal&amp;tmpl=component&amp;field=' . $idTag . '" title="' . JText::_('JSELECT') . '">' . JText::_('JSELECT') . '</a>';

		$document = JFactory::getDocument();
		$document->addScriptDeclaration(
				"\n\n// Extly's userControl\n\twindow.jSelectUser_{$idTag} = function jSelectUser_author(id, username) {
		jQuery('#{$idTag}').val(id);
		jQuery('#{$idTag}_username').val(username);
		try {
			jQuery('#sbox-window').close();
		} catch (err) {
			SqueezeBox.close();
		}
	};\n\n"
		);

		return self::genericControl($label, $desc, $name, $control);
	}

	/**
	 * accessLevelControl
	 *
	 * @param   string  $selected  Value
	 * @param   string  $name      The name for the field
	 * @param   string  $label     Label
	 * @param   string  $desc      Description
	 * @param   array   $idTag     Additional HTML attributes for the <select> tag
	 * @param   string  $class     Class
	 *
	 * @return  string  HTML
	 */
	public static function accessLevelControl($selected, $name, $label, $desc, $idTag = null, $class = null)
	{
		if (!$idTag)
		{
			$idTag = self::generateIdTag();
		}

		$attr = array();

		if (!empty($class))
		{
			$attr['class'] = $class;
		}

		$control = JHtml::_('access.level', $name, $selected, $attr, null, $idTag);

		return self::genericControl($label, $desc, $name, $control);
	}

	/**
	 * numericUnitsControl
	 *
	 * @param   string  $selectedNumeric  Value
	 * @param   string  $nameNumeric      The name for the field
	 * @param   string  $selectedUnit     Value
	 * @param   string  $nameUnit         The name for the field
	 * @param   string  $units            The name for the field
	 * @param   string  $label            Label
	 * @param   string  $desc             Description
	 * @param   array   $idTag            Additional HTML attributes for the <select> tag
	 * @param   string  $class            Class
	 *
	 * @return  string  HTML
	 */
	public static function numericUnitsControl($selectedNumeric, $nameNumeric, $selectedUnit, $nameUnit, $units, $label, $desc, $idTag = null, $class = null)
	{
		if (!$idTag)
		{
			$idTag = self::generateIdTag();
		}

		$bogusNameTag = self::generateIdTag();

		if (!empty($class))
		{
			$class = 'class="' . $class . '"';
		}

		$control = '<input type="text" name="' . $nameNumeric . '" id="' . $idTag . '" value="' . $selectedNumeric . '" ' . $class . '/> &nbsp;';
		$control .= EHtmlSelect::btnGroupList($selectedUnit, $nameUnit, array(), $units, $idTag . '_units');

		return self::genericControl($label, $desc, $nameNumeric, $control);
	}
}
